@@ -145,9 +145,12 @@ def read_pkg_file(self, file):
 
 
 def single_line(val):
-    # quick and dirty validation for description pypa/setuptools#1390
+    """
+    Quick and dirty validation for Summary pypa/setuptools#1390.
+    """
     if '\n' in val:
-        # TODO after 2021-07-31: Replace with `raise ValueError("newlines not allowed")`
+        # TODO: Replace with `raise ValueError("newlines not allowed")`
+        # after reviewing #2893.
         warnings.warn("newlines not allowed and will break in the future")
         val = val.strip().split('\n')[0]
     return val
